# coding=utf-8

__author__ = 'giz'

import cPickle
import os
import itertools
import math
import re
from collections import defaultdict

import nltk
from nltk import bigrams
from nltk import trigrams

pathToCollection = "./open/"
pathToModel = "./models/model.pkl"


numberUnigrams = 1
numberBigrams = 4
numberTrigrams = 1

class Text():
    def __init__(self, text):
        words = nltk.PunktWordTokenizer().tokenize(text)
        words = filter(lambda x: len(x) > 3 and re.match('[a-zA-Zа-яА-Я0-9йЙёЁ\s\-\'"]+$'.decode('utf-8'), x), words)
        words = words[:250]
        stemmer = nltk.stem.snowball.RussianStemmer()
        self.words = []
        self.originals = {}
        for word in words:
            stem_word = stemmer.stem(word)
            self.words.append(stem_word)
            if not self.originals.has_key(word):
                self.originals[stem_word] = word


class Trainer():
    model = None

    def __init__ (self, pathToCollection, pathToModel):
        self.pathToCollection = pathToCollection
        self.pathToModel = pathToModel
        if not(self.loadModel()):
            self.model = self.computeIDF()
            self.saveModel()

    def computeIDF(self):
        docs = defaultdict(float)
        files = os.listdir(self.pathToCollection)
        for file_name in files:
            file = open(os.path.join(self.pathToCollection, file_name))
            text = Text(unicode(file.read(), "utf-8"))

            tokens = set(word for word in text.words)
            bi_tokens = set(' '.join(token) for token in bigrams(tokens))
            tri_tokens = set(' '.join(token) for token in trigrams(tokens))
            for token in itertools.chain(tokens, bi_tokens, tri_tokens):
                docs[token] += docs.setdefault(token, 0.0) + 1.0

        idf = {}
        for token in docs:
            if docs[token] > 1:
                idf[token] = -math.log(docs[token] / len(files))
        return idf, len(files)

    def saveModel(self):
        f = open(self.pathToModel, "wb")
        cPickle.dump(self.model, f)
        f.close()

    def loadModel(self):
        try:
            f = open(self.pathToModel)
            self.model = cPickle.load(f)
            f.close()
            return True
        except:
            return False

    def getModel(self):
        return self.model



class Keywords():

    def __init__(self):
        # load trained model
        self.model = Trainer(pathToCollection, pathToModel).getModel()

    def weightingScheme(self, lst, model):
        return [(name, count * model[0].get(name, -math.log(1.0 / model[1])))
                for name, count, factor in lst if factor < 0.3]

    def top(self, lst, n):
        return map(lambda x: x[0], sorted(lst, key = lambda x: -x[1])[:n])

    def objectCounter(self, lst):
        objects = map(lambda x: x[1], lst)
        poses = {}
        len_objects = len(objects)
        for pos, elem in lst:
            if elem not in poses:
                poses[elem] = pos / float(len_objects)

        return [(obj, objects.count(obj), poses[obj]) for obj in set(objects)]

    def getKeywords(self, text):
        # text processor
        text = Text(text)
        tokens = list(enumerate(text.words))
        bi_tokens = list(enumerate(' '.join(token) for token in bigrams(text.words)))
        tri_tokens = list(enumerate(' '.join(token) for token in trigrams(text.words)))

        def restore(ngram):
            tokens = ngram.split()
            return ' '.join(text.originals[t] for t in tokens if text.originals.get(t))

        unigramWords = map(restore,
            self.top(self.weightingScheme(self.objectCounter(tokens), self.model), numberUnigrams))
        bigramWords = map(restore,
            self.top(self.weightingScheme(self.objectCounter(bi_tokens), self.model), numberBigrams))
        trigramWords = map(restore,
            self.top(self.weightingScheme(self.objectCounter(tri_tokens), self.model), numberTrigrams))
        return unigramWords + bigramWords + trigramWords
#
#kw = Keywords()
#f = open("../open/data/isp_20_2011_283.txt")
#text = unicode(f.read(), "utf-8")
#f.close()
#keywords = kw.getKeywords(text)
#print repr(keywords).decode("unicode_escape").encode("utf-8")

